import numpy as np
import matplotlib.pyplot as plt
import matplotlib.transforms as transforms
from matplotlib.patches import Rectangle, Ellipse

from tools import *

def plot_profiles(self):

    d_prof,d_cen,c_prof,c_cen = self.model_args

    fig,ax = plt.subplots(1,1,figsize=(15,6),dpi=75,tight_layout=True)

    ax.axhline(continuum(self.xf,self.subpx_pars),c='k',ls='--',lw=1.5,label='Continuum')
    ax.plot(self.xf,direct_model(self.xf,self.subpx_pars,d_prof,d_cen),c='g',lw=1.5,label='Main Object')
    ax.plot(self.xf,contam_model(self.xf,self.subpx_pars,c_prof,c_cen,self.prior1D),c='b',lw=1.5,label='Contaminants')
    ax.plot(self.xf,profile_model(self.xf,self.subpx_pars,self.model_args,self.prior1D),lw=1.5,c='r',label='Model')
    ax.plot(self.xf,self.grism1D_img,c='k',lw=1.5,label='Observed')
    ax.fill_between(self.xf,self.grism1D_img-self.grism1D_err,self.grism1D_img+self.grism1D_err,color='k',lw=0,alpha=0.1)

    trans = transforms.blended_transform_factory(ax.transData, ax.transAxes)
    ax.text(d_cen+self.subpx_pars[2],0.955,"%i"%self.entry['OBJ_NUM'],color='g',fontsize=12,va='bottom',ha='center',transform=trans)
    ax.vlines(d_cen+self.subpx_pars[2],0.85,0.95,color='g',lw=1.5,transform=trans)
    for x,xc,subpx in zip(self.grism_contams,c_cen,self.subpx_pars[4::2]):
        if min(self.xf)<xc+subpx<max(self.xf):
            ax.text(xc+subpx,0.955,"%i"%x['OBJ_NUM'],color='b',fontsize=12,va='bottom',ha='center',transform=trans)
            ax.vlines(xc+subpx,0.85,0.95,color='b',lw=1.5,transform=trans)

    sig = np.std(self.grism1D_img)
    sig = np.std(np.clip(self.grism1D_img,-3*sig,3*sig))
    ymin, ymax = -3*sig, 1.5*np.max(self.grism1D_img)

    ax.set_xlabel('Pixels')
    ax.set_ylabel('Counts/s')
    ax.set_xlim(min(self.xf),max(self.xf))
    ax.set_ylim(ymin,ymax)
    ax.legend(fontsize=12,loc='best')
    return fig

def plot_grisms(self):

    fig,(ax1,ax2,ax3,ax4) = plt.subplots(4,1,figsize=(10,10))
    fig.subplots_adjust(left=0.05,right=0.95,bottom=0.15,top=0.98,hspace=0)

    dx = 2*self.grism_img.shape[1]/self.pixfrac_corr
    dy = 2*self.grism_img.shape[0]/self.pixfrac_corr
    stamp = cut_direct(self.direct_img,self.entry,dx=dx,dy=dy)
    c,r = np.indices(stamp.shape)
    c,r = c-stamp.shape[0]/2., r-stamp.shape[1]/2.

    stamp_ = stamp[(-5*self.entry['BBOX_X']<r)&(r<5*self.entry['BBOX_X'])&(-5*self.entry['BBOX_Y']<c)&(c<5*self.entry['BBOX_Y'])]
    sig = np.std(stamp_)
    sig = np.std(np.clip(stamp_,-5*sig,5*sig))

    ax1.pcolormesh(r,c,stamp,cmap=plt.cm.viridis,vmin=-2*sig,vmax=2*sig)
    #ax1.add_patch(Ellipse((0,0), 4*self.entry['A_IMAGE'], 4*self.entry['B_IMAGE'], self.entry['THETA_IMAGE'], linewidth=1, facecolor='none', edgecolor='w'))
    ax1.text(self.entry['BBOX_X'],self.entry['BBOX_Y'],"%i"%self.entry['OBJ_NUM'],fontsize=14,fontweight=500,color='w')
    for contam in self.grism_contams:
        x,y = contam['X_IMAGE'] - self.entry['X_IMAGE'], contam['Y_IMAGE'] - self.entry['Y_IMAGE']
        ax1.add_patch(Ellipse((x,y), 4*contam['A_IMAGE'], 4*contam['B_IMAGE'], contam['THETA_IMAGE'], linewidth=1, facecolor='none', edgecolor='k'))
        ax1.text(x+self.entry['BBOX_X'],y+self.entry['BBOX_Y'],"%i"%contam['OBJ_NUM'],fontsize=14,fontweight=500,color='k')

    ax1.add_patch(Rectangle((-dx/4.,-dy/4.),dx/2.,dy/2.,linewidth=1,linestyle='--',facecolor='none',edgecolor='k'))
    ax1.set_xlim(-dx/2.,+dx/2.)
    ax1.set_ylim(-dy/2.,+dy/2.)
    ax1.xaxis.set_visible(False)

    c,r = np.indices(self.grism_img.shape)
    c = c - self.grism_img.shape[0]/2.
    r = get_waves(r,self.grism_hdr)
    cond = (-2*self.entry['BBOX_Y']*self.pixfrac_corr<c) & (c<2*self.entry['BBOX_Y']*self.pixfrac_corr)
    sig = np.std(self.grism_img[cond])
    sig = np.std(np.clip(self.grism_img[cond],-5*sig,5*sig))

    im = ax2.pcolormesh(r,c,self.grism_img,vmin=-2*sig,vmax=2*sig,cmap=plt.cm.viridis)
    ax3.pcolormesh(r,c,self.grism_model,vmin=-2*sig,vmax=2*sig,cmap=plt.cm.viridis)
    ax4.pcolormesh(r,c,self.grism_clean,vmin=-2*sig,vmax=2*sig,cmap=plt.cm.viridis)

    cbaxes = fig.add_axes([0.05, 0.05, 0.9, 0.02])
    cbax   = fig.colorbar(mappable=im, cax=cbaxes, orientation='horizontal')

    if   self.grism == 'G102':
        xc, dx = 8000., 11800. - 8000.
        _ = [ax.set_xlim(7250,12750) for ax in [ax2,ax3,ax4]]
    elif self.grism == 'G141':
        xc, dx = 10500., 17000. - 10500.
        _ = [ax.set_xlim(9500,18500) for ax in [ax2,ax3,ax4]]

    ax4.set_xlabel('Wavelength [$\AA$]')
    _ = [ax.set_ylim(-self.grism_img.shape[0]/2.,self.grism_img.shape[0]/2.) for ax in [ax2,ax3,ax4]]
    _ = [ax.yaxis.set_visible(False) for ax in [ax1,ax2,ax3,ax4]]

    yc, dy = 0, 2*self.entry['BBOX_Y']*self.pixfrac_corr
    for ax in [ax2,ax3,ax4]:
        #ax.add_patch(Rectangle((xc,yc-dy/2.+self.subpx_pars[2]), dx, dy,linewidth=1,facecolor='none',edgecolor='w'))
        for contam,subpx in zip(self.grism_contams,self.subpx_pars[4::2]):
            rxc = xc + (contam['X_IMAGE'] - self.entry['X_IMAGE'])*self.pixfrac_corr * self.grism_hdr['CDELT1']
            ryc = yc + (contam['Y_IMAGE'] - self.entry['Y_IMAGE'])*self.pixfrac_corr
            rdy = 2*contam['BBOX_Y']*self.pixfrac_corr
            ax.add_patch(Rectangle((rxc,ryc-rdy/2.+subpx), dx, dy,linewidth=1,facecolor='none',edgecolor='k'))

    return fig
