import os
import numpy as np
import astropy.io.fits as fitsio
import matplotlib.pyplot as plt

from tools import *

px_scale_grism = 0.128254
px_scale_direct = 0.08
px_scale_cor = px_scale_direct / px_scale_grism

sens_fn = {}
sens_G102 = fitsio.getdata('config/WFC3.G102.1st.sens.fits')
sens_fn['G102'] = scipy.interpolate.interp1d(sens_G102['WAVELENGTH'],sens_G102['SENSITIVITY'],bounds_error=False,fill_value=0)
sens_G141 = fitsio.getdata('config/WFC3.G141.1st.sens.fits')
sens_fn['G141'] = scipy.interpolate.interp1d(sens_G141['WAVELENGTH'],sens_G141['SENSITIVITY'],bounds_error=False,fill_value=0)

def calc_1d_spec(entry,subpx_entry,grism,extname,aXe_output=False):

    fname = '%s/Par%i/clean/aXeWFC3_%s_clean_ID%i.fits' % (output_dir,entry['PAR_NUM'],grism,entry['NUMBER'])

    spec_2d,hdr = fitsio.getdata(fname,extname=extname,header=True)
    spec_2d[~np.isfinite(spec_2d)] = 0

    if aXe_output:

        d_prof = fitsio.getdata(data_dir+'Par%i/%s_DRIZZLE/aXeWFC3_%s_mef_ID%i.fits'%(entry['PAR_NUM'],grism,grism,entry['NUMBER']),extname='MOD')
        contam = fitsio.getdata(data_dir+'Par%i/%s_DRIZZLE/aXeWFC3_%s_mef_ID%i.fits'%(entry['PAR_NUM'],grism,grism,entry['NUMBER']),extname='CON')
        spec_1d = np.sum(spec_2d*d_prof, axis=0)

    else:

        assert entry['PAR_NUM']==subpx_entry['PAR'] and entry['NUMBER']==subpx_entry['OBJ']
        subpx_pars = [subpx_entry['conti'],subpx_entry['d_amp'],subpx_entry['d_off']]

        xx = np.arange(spec_2d.shape[0])
        d_prof_fn = mk_stamp_profile(entry,px_scale_cor,output_dir,entry['DIRECT_%s'%grism])
        d_cen  = spec_2d.shape[0] / 2.
        d_prof = direct_model(xx,subpx_pars,d_prof_fn,d_cen)
        norm = scipy.integrate.quad(d_prof_fn,0,spec_2d.shape[0],epsabs=1e-6,epsrel=1e-6,limit=100)[0]
        d_prof = d_prof / norm

        spec_1d = np.zeros(spec_2d.shape[1])
        for i in range(spec_2d.shape[1]): spec_1d[i] = np.sum(spec_2d[:,i] * d_prof) - subpx_entry['conti']

    wave = np.arange(spec_2d.shape[1])+1
    wave = (wave-hdr['CRPIX1'])*hdr['CDELT1'] + hdr['CRVAL1']
    sens = sens_fn[grism](wave)
    cond = (sens!=0)
    flux_1d = np.zeros(len(spec_1d))
    flux_1d[cond] = spec_1d[cond] / sens[cond]
    flux_1d[~cond] = 0.

    return wave, spec_1d, flux_1d

def write_1d_spec(sample,subpx_102,subpx_141):

    for entry,subpx_entry_G102,subpx_entry_G141 in zip(sample,subpx_102,subpx_141):
        print entry['PAR_NUM'],entry['NUMBER']
        for grism,subpx_entry in zip(['G102','G141'],[subpx_entry_G102,subpx_entry_G141]):
            wave, spec_1d, flux_1d = calc_1d_spec(entry,subpx_entry,grism,extname='CLEAN',aXe_output=False)
            err_1d = np.genfromtxt('/data/highzgal/PUBLICACCESS/WISPS/data/V5.0/Par%i/Spectra/Par%i_%s_BEAM_%iA.dat' % (entry['PAR_NUM'],entry['PAR_NUM'],grism,entry['NUMBER']),usecols=[2,],unpack=True)
            np.savetxt(output_dir+'/clean_1D/Par%i_%s_BEAM_%iA_clean.dat' % (entry['PAR_NUM'],grism,entry['NUMBER']), np.vstack((wave, spec_1d, flux_1d, err_1d)).T, fmt="%15.2f%15.6e%15.6e%15.6e", header="%13s%15s%15s%15s"%("Wavelength","Counts/sec","Flux","Flux_err"))

if __name__ == '__main__':

    data_dir = '/data/highzgal/PUBLICACCESS/WISPS/data/V5.0/'
    output_dir = '/data/highzgal/mehta/WISP/cleaning_tony/output/'

    short_list = np.genfromtxt('fin_sample.dat',dtype=[('PAR_NUM',int),('NUMBER',int),('HMAG',float),('DIRECT_G102','|S4'),('DIRECT_G141','|S4')])
    subpx_102 = np.genfromtxt(output_dir+'/subpx_offsets_G102.dat',dtype=[('PAR',int),('OBJ',int),('conti',float),('d_amp',float),('d_off',float)],usecols=(0,1,2,3,4))
    subpx_141 = np.genfromtxt(output_dir+'/subpx_offsets_G141.dat',dtype=[('PAR',int),('OBJ',int),('conti',float),('d_amp',float),('d_off',float)],usecols=(0,1,2,3,4))

    short_list = np.sort(short_list,order=['PAR_NUM','NUMBER'])
    subpx_102 = np.sort(subpx_102,order=['PAR','OBJ'])
    subpx_141 = np.sort(subpx_141,order=['PAR','OBJ'])

    write_1d_spec(short_list,subpx_102,subpx_141)

    # i = 50
    # entry, subpx_entry, grism = short_list[i], subpx_141[i], 'G141'
    # wave, spec_1d, flux_1d = calc_1d_spec(entry,subpx_entry,grism=grism,extname='SCI',aXe_output=True)
    # plt.plot(wave,flux_1d,c='b')
    # wave, spec_1d, flux_1d = calc_1d_spec(entry,subpx_entry,grism=grism,extname='CLEAN',aXe_output=False)
    # plt.plot(wave,flux_1d,c='r')
    # wave, flux_1d = np.genfromtxt('/data/highzgal/PUBLICACCESS/WISPS/data/V5.0/Par%i/Spectra/Par%i_%s_BEAM_%iA.dat' % (entry['PAR_NUM'],entry['PAR_NUM'],grism,entry['NUMBER']),usecols=[0,1],unpack=True)
    # plt.plot(wave,flux_1d,c='k')
    # plt.show()